use std::fs;

pub fn solve() {
    let path = "res/10/input.txt";

    let contents = fs::read_to_string(path).expect("I/O error, wrong path?");

    let lines: Vec<&str> = contents.lines().filter(|line| *line != "").collect();

    let mut reg_x = 1;
    let mut cycle = 0;
    let mut signal_strengths = Vec::<i32>::new();

    let mut crt = Vec::<bool>::new();

    // parse commands line by line
    lines.iter().for_each(|line| {
        let parts: Vec<&str> = line.split_whitespace().collect();
        match parts[0] {
            "noop" => {
                // increase cycle 1 time
                cycle += 1;
                if rel_cycle(cycle) {
                    signal_strengths.push(cycle * reg_x);
                }
                crt.push(sprite_draw(cycle, reg_x));
            }
            "addx" => {
                // increase cycle 2 times
                (0..2).for_each(|_| {
                    cycle += 1;
                    if rel_cycle(cycle) {
                        signal_strengths.push(cycle * reg_x);
                    }
                    crt.push(sprite_draw(cycle, reg_x));
                });
                // change regX according to parameter
                reg_x += parts[1].parse::<i32>().unwrap();
            }
            _ => {}
        }
    });

    let res: i32 = signal_strengths.iter().sum();

    println!("Result 1: {}", res);

    println!("Result 2: ");
    let mut print_buffer = "".to_string();
    // print the crt line by line
    crt.iter().enumerate().for_each(|(i, p)| {
        if *p {
            print_buffer += "#"
        } else {
            print_buffer += "."
        }

        // print buffer if a line is completed
        if (i + 1) % 40 == 0 {
            println!("{}", print_buffer);
            print_buffer = "".to_string();
        }
    })
}

// returns whether the passed cycle is a relevant one
fn rel_cycle(cycle: i32) -> bool {
    if (cycle - 20) % 40 == 0 && cycle >= 20 {
        true
    } else {
        false
    }
}

// returns whether the sprite is currently in a position to draw a pixel
fn sprite_draw(cycle: i32, reg_x: i32) -> bool {
    return ((cycle - 1) % 40).abs_diff(reg_x) <= 1;
}
