use serde_json::Value;
use std::{cmp::Ordering, fs};

pub fn solve() {
    let path = "res/13/input.txt";

    let contents = fs::read_to_string(path).expect("I/O error, wrong path?");

    // Task 1
    let pairs: Vec<(&str, &str)> = contents
        .split("\n\n")
        .map(|pair| pair.split_at(pair.find("\n").unwrap()))
        .collect();

    let mut right_order_indices = Vec::<u32>::new();
    pairs.iter().enumerate().for_each(|(i, pair)| {
        let json1: Value = serde_json::from_str(pair.0).unwrap();
        let json2: Value = serde_json::from_str(pair.1).unwrap();

        // println!("== Pair {} ==", i+1);
        let res = is_right_order((&json1, &json2));

        // println!("Pair {}: {res:?}\n", i+1);

        if res != Ordered::No {
            right_order_indices.push((i + 1) as u32);
        }
    });

    let res: u32 = right_order_indices.iter().sum();
    println!("Result 1: {res}");

    // Task 2
    let mut packets: Vec<Value> = contents
        .lines()
        .filter(|line| *line != "")
        .map(|line| serde_json::from_str(line).unwrap())
        .collect();

    // add [[2]] and [[6]] as divider packets
    packets.push(serde_json::from_str("[[6]]").unwrap());
    packets.push(serde_json::from_str("[[2]]").unwrap());

    // sort by compare function using is_right_order
    packets.sort_by(|a, b| match is_right_order((a, b)) {
        Ordered::Yes => Ordering::Less,
        Ordered::Maybe => Ordering::Equal,
        Ordered::No => Ordering::Greater,
    });

    // find [[2]] and [[6]] indices
    let mut i2 = 0;
    let mut i6 = 0;
    packets.iter().enumerate().for_each(|(i, v)| {
        if v.is_array() && v.as_array().unwrap().len() == 1 {
            let v = v.as_array().unwrap();
            if v[0].is_array() && v[0].as_array().unwrap().len() == 1 {
                let v = v[0].as_array().unwrap();
                if v[0].is_number() && v[0].as_u64().unwrap() == 2 {
                    i2 = i + 1
                } else if v[0].is_number() && v[0].as_u64().unwrap() == 6 {
                    i6 = i + 1
                }
            }
        }
    });

    let res = i2 * i6;

    println!("Result 2: {res}");
}

#[derive(PartialEq, Debug)]
enum Ordered {
    Yes,
    Maybe,
    No,
}

// returns whether the passed Values are in the right order
// with Ordered enum option
fn is_right_order(values: (&Value, &Value)) -> Ordered {
    //println!("Compare {} vs {}", values.0, values.1);

    // both values numbers
    if values.0.is_number() && values.1.is_number() {
        let v0 = values.0.as_u64().unwrap();
        let v1 = values.1.as_u64().unwrap();
        // wrong order if right is higher than left
        if v0 < v1 {
            return Ordered::Yes;
        } else if v0 == v1 {
            return Ordered::Maybe;
        } else {
            return Ordered::No;
        }
    }
    // left number, right array
    else if values.0.is_number() && values.1.is_array() {
        // wrap number into arr, then compare again
        let left = Value::Array(vec![values.0.clone()]);
        return is_right_order((&left, &values.1));
    }
    // left array, right number
    else if values.0.is_array() && values.1.is_number() {
        // wrap number into arr, then compare again
        let right = Value::Array(vec![values.1.clone()]);
        return is_right_order((&values.0, &right));
    }
    // both array
    else if values.0.is_array() && values.1.is_array() {
        let a0 = values.0.as_array().unwrap();
        let a1 = values.1.as_array().unwrap();

        let mut a0iter = a0.iter();
        let mut a1iter = a1.iter();

        loop {
            let left = a0iter.next();
            let right = a1iter.next();

            // left smaller, counts as ordered
            if left.is_none() && right.is_some() {
                return Ordered::Yes;
            }

            // right smaller, counts as unordered
            if left.is_some() && right.is_none() {
                return Ordered::No;
            }

            // both have same length, doesn't say anything about ordering
            if left.is_none() && right.is_none() {
                return Ordered::Maybe;
            }

            let left = left.unwrap();
            let right = right.unwrap();

            // only go into next iteration if not definitely yes or no
            let res = is_right_order((&left, &right));
            if res != Ordered::Maybe {
                return res;
            }
        }
    }

    Ordered::Yes
}
